const electron = require('electron');
const { app, BrowserWindow } = require('electron');
const path = require('path');
const url = require('url');
const { ipcEvents } = require('./constants');
const { setMainMenu } = require('./main-menu.js'); // TODO: fix and re-use

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let win;
const TITLE = 'AB Tasty Widget Tester';

function createWindow() {
  const mainScreen = electron.screen.getPrimaryDisplay();
  const { width, height } = mainScreen.size;
  win = new BrowserWindow({
    width,
    height,
    title: TITLE,
    show: false,
    webPreferences: {
      webSecurity: false,
      allowRunningInsecureContent: true,
    },
  });
  // and load the index.html of the app.
  win.loadURL(
    url.format({
      pathname: path.join(__dirname, 'index.html'),
      protocol: 'file:',
      slashes: true,
    })
  );

  win.on('ready-to-show', () => win.show());
  win.on('closed', () => {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    win = null;
  });
}

const shouldQuit = app.makeSingleInstance(() => {
  // Someone tried to run a second instance, we should focus our window.
  if (win) {
    if (win.isMinimized()) win.restore();
    win.focus();
  }
});

if (shouldQuit) app.quit();

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', () => {
  createWindow();
  win.webContents.on('did-finish-load', () => {
    win.webContents.send(ipcEvents.loadWebpage, process.argv[2]); // todo : destructuring at the top of the file
    win.webContents.send(ipcEvents.loadWidget, process.argv[3]);
  });
  // setMainMenu(win);
});

// On macOS it is common for applications and their menu bar
// to stay active until the user quits explicitly with Cmd + Q
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') app.quit();
});

// On macOS it's common to re-create a window in the app when the
// dock icon is clicked and there are no other windows open.
app.on('activate', () => {
  if (win === null) createWindow();
});
