const { ipcRenderer } = require('electron');
const JSONfn = require('json-fn');
const { debounce } = require('../helpers');
const { ipcEvents } = require('../constants');

// TODO: remove all of jQuery

function getSubmitFormButton() {
  const button = document.createElement('input');
  button.type = 'submit';
  ['btn', 'btn-primary', 'pull-right', 'btn-large'].map(className =>
    button.classList.add(className)
  );
  return button;
}

ipcRenderer.on('setForm', (event, formContent, isHtml, isReload) => {
  const formPlaceholder = '#form-placeholder';
  let unmountWidgetForm;
  let formContentFn = JSONfn.parse(formContent);
  // Babel 6 exports default as { default: [] } instead of just [].
  // See https://github.com/webpack/webpack/issues/706.
  if (typeof formContentFn === 'object' && formContentFn.default) {
    formContentFn = formContentFn.default;
  }
  const refreshWebsiteView = (formData = '') => {
    if (isHtml) formData = _parsePluginOptions($('form'), []);
    ipcRenderer.sendToHost('re-render-view', formData);
  };

  if (isReload) {
    $('#form-placeholder').html('');
  }

  // Append form
  if (isHtml) {
    $(formPlaceholder).append(formContentFn);
    refreshWebsiteView(); // refresh widget with initial form config

    // Handle submit
    const submitFormButton = getSubmitFormButton();
    $('form').append(submitFormButton);
    document.querySelector('form').onsubmit = refreshWebsiteView;
  } else {
    const submitText = {
      en: 'Submit',
      fr: 'Soumettre',
      de: 'Absenden',
      es: 'Enviar',
    };
    const mount = require('@abtasty/widget-form-generator');
    const config = {
      container: document.querySelector(formPlaceholder),
      form: formContentFn,
      values: {},
      onValueChange: res => {
        refreshWebsiteView(res);
      },
      onSubmit: res => {
        refreshWebsiteView(res);
      },
      selectElement: () => getSelectedElement(),
      mediaUpload: () => {},
      lang: 'en',
      isElectron: true,
      submitText: submitText.en,
    };
    unmountWidgetForm = mount(config);
  }

  // Select for langs
  if (!isHtml) {
    let selectInput = '<select id="lang">';
    const tabLang = ['fr', 'en', 'es', 'de'];
    tabLang.forEach(lang => {
      selectInput += `<option value="${lang}">${lang}</option>`;
    });
    selectInput += '</select>';
    $(formPlaceholder).before(selectInput);
    document.getElementById('lang').value = 'en';
  }

  // Handle switch language
  if (!isHtml) {
    document.getElementById('lang').addEventListener('change', function() {
      if (typeof unmountWidgetForm === 'function') {
        unmountWidgetForm();
      }
      // document.querySelector(formPlaceholder).childNodes[0].remove();
      const submitText = {
        en: 'Submit',
        fr: 'Soumettre',
        de: 'Absenden',
        es: 'Enviar',
      };
      const mount = require('@abtasty/widget-form-generator');
      const config = {
        container: document.querySelector(formPlaceholder),
        form: formContentFn,
        values: {},
        onValueChange: res => {
          refreshWebsiteView(res);
        },
        onSubmit: res => {
          refreshWebsiteView(res);
        },
        selectElement: () => getSelectedElement(),
        mediaUpload: () => {},
        lang: this.value,
        isElectron: true,
        submitText:
          typeof submitText[this.value] !== 'undefined' ? submitText[this.value] : submitText.en,
      };
      mount(config);
    });
  }

  // Special behaviour for the "selectElement" React component inside the widget-cli
  const getSelectedElement = () => {
    ipcRenderer.sendToHost(ipcEvents.selectElement, '');
    return new Promise(resolve =>
      ipcRenderer.on(ipcEvents.selectElementComplete, (e, selector) => resolve(selector))
    );
  };
});
