const tldList = require('tld-list');

const SPINNING_CLASS = 'spinning-animation';

function debounce(func, wait, immediate) {
  let timeout;
  return () => {
    const context = this;
    const args = arguments;
    const later = () => {
      timeout = null;
      if (!immediate) func.apply(context, args);
    };
    const callNow = immediate && !timeout;
    clearTimeout(timeout);
    timeout = setTimeout(later, wait);
    if (callNow) func.apply(context, args);
  };
}

function domReady(fn) {
  if (
    document.attachEvent ? document.readyState === 'complete' : document.readyState !== 'loading'
  ) {
    fn();
  } else {
    document.addEventListener('DOMContentLoaded', fn);
  }
}

function injectJquery() {
  window.$ = window.jQuery = require('jquery');
}

const startSpinningAnimation = el => {
  el.className += ` ${SPINNING_CLASS}`;
};

const stopSpinningAnimation = el => {
  el.classList.remove(SPINNING_CLASS);
};

function isValidUrl(url) {
  const tldListRegex = tldList.map(str => `\\.${str}`).join('|');
  const regex = `(${tldListRegex})(?:\\/|#|\\?|$)`;
  return url.match(regex) !== null;
}

function getPosition(position, obj) {
  const attributes = {
    left: 'offsetLeft',
    top: 'offsetTop',
  };
  const attr = attributes[position];
  let curleft = 0;
  let elm = obj;
  if (elm.offsetParent) {
    curleft = elm[attr];
    while ((elm = elm.offsetParent)) {
      curleft += elm[attr];
    }
  }
  return curleft;
}

function addProtocolToUrl(url) {
  let newUrl = url;
  if (url.indexOf('http') === -1) {
    newUrl = `http://${url}`;
  }
  return newUrl;
}

module.exports = {
  debounce,
  startSpinningAnimation,
  stopSpinningAnimation,
  isValidUrl,
  addProtocolToUrl,
  domReady,
  injectJquery,
  getPosition,
};
