const path = require('path');
const fs = require('fs');
const ipcRenderer = require('electron').ipcRenderer;

const { stopSpinningAnimation } = require('./helpers');
const webpage = require('./webpage/controller');
const form = require('./form/controller');

const { $WEBVIEW__WEBPAGE, $WEBVIEW__CONFIGFORM, ipcEvents } = require('./constants');

let data;

$WEBVIEW__CONFIGFORM.addEventListener('ipc-message', ({ args, channel }) => {
  if (channel === 're-render-view') {
    data = args[0];
    webpage.reload();
  } else if (channel === ipcEvents.selectElement) {
    $WEBVIEW__WEBPAGE.send(channel);
  }
});

// transmit every raw IPC message to the webpage
// ipc message are sent under this format : ipcRenderer.sendToHost('select-element', '');
webpage.events.electron.onMessage(({ args, channel }) =>
  $WEBVIEW__CONFIGFORM.send(channel, args[0])
);

ipcRenderer.on(ipcEvents.loadWidget, (event, widgetPath) => {
  const pkg = require(path.resolve(widgetPath, 'package.json'));
  const mainFilePath = path.resolve(widgetPath, pkg.main);
  const formPath = path.resolve(widgetPath, pkg.form);
  const widgetPackageName = pkg.name;
  const isHtml = formPath.endsWith('html');
  let formContent;

  const prepareHtmlForm = widgetForm =>
    `<form class="ab-embed-editor__plugins-library__plugin" data-plugin-id="1" action="javascript:;">${widgetForm}</form>`;

  formContent = isHtml ? prepareHtmlForm(fs.readFileSync(formPath, 'utf8')) : require(formPath);

  // Add the form content to the form webview.
  form.events.domready(() => form.injectForm(formContent, isHtml, false));
  form.events.click.toggler(form.toggleVisibility);

  webpage.events.domready(() => {
    stopSpinningAnimation(document.querySelector('#refresh'));
    webpage.injectWidget(mainFilePath, widgetPackageName, data);
    webpage.fillUrlInput($WEBVIEW__WEBPAGE.src);
  });

  webpage.events.failLoading(() => {
    // TODO Afficher une erreur sur la webview -> Maquette ?
    stopSpinningAnimation(document.querySelector('#refresh'));
  });

  // Delete require cache
  const clearRequireCache = () => {
    // TODO : why this line exists ?
    Object.keys(require.cache).forEach(key => delete require.cache[key]);
  };

  // Watch form file
  fs.watch(mainFilePath, webpage.reload);
  if (isHtml) {
    fs.watch(formPath, () => {
      clearRequireCache();
      formContent = prepareHtmlForm();
      form.injectForm(formContent, isHtml, true);
    });
  } else {
    fs.watch(formPath, () => clearRequireCache());
  }
});

ipcRenderer.on(ipcEvents.loadWebpage, (event, url) => {
  $WEBVIEW__WEBPAGE.src = url;
  webpage.fillUrlInput(url);
});

form.events.click.devtools(form.openDevTools);

// USER EVENTS HANDLERS INTERFACE (webpage)
webpage.resetSizeInputs();
webpage.events.click.devtoolsButton(webpage.openDevTools);
webpage.events.click.prevButton(webpage.prev);
webpage.events.click.nextButton(webpage.next);
webpage.events.click.refreshButton(webpage.reload);
webpage.events.click.landscapeOrientation(() => webpage.rotate('portrait'));
webpage.events.click.portraitOrientation(() => webpage.rotate('landscape'));
webpage.events.change.url(e => webpage.redirect(e.target.value));
webpage.events.change.deviceDropdown(webpage.resizeWebpageByDevice);
webpage.events.change.widthInput(webpage.updateSize);
webpage.events.change.heightInput(webpage.updateSize);
webpage.events.resize.webview(webpage.resetSizeInputs);
