const { app, Menu, dialog, shell } = require('electron');

const isWindows = process.platform === 'win32';

module.exports = {
  setMainMenu,
};

function askToQuit() {
  dialog.showMessageBox(
    {
      type: 'question',
      buttons: ['no', 'yes'],
      cancelId: 0,
      title: 'Have a good day !',
      message: 'Are you sure you want to quit ?',
    },
    response => {
      if (response === 1) app.quit();
    }
  );
}

function openFormConfig() {
  dialog.showOpenDialog(
    {
      title: 'Select your form configuration file',
      defaultPath: process.cwd(),
      properties: ['openFile'],
      filters: [
        {
          name: 'Html configuration',
          extensions: ['html'],
        },
        {
          name: 'JS configuration',
          extensions: ['js'],
        },
      ],
      message: 'Select your form configuration file',
    },
    response => {
      if (response === undefined) {
        // console.log(response);
      } else {
        shell.openItem(response[0]);
      }
    }
  );
}

function setMainMenu(win) {
  const name = 'AB Tasty Widget Tester';

  const template = [
    {
      label: isWindows ? 'File' : name,
      submenu: [
        {
          label: isWindows ? 'Exit' : `Quit ${name}`,
          accelerator: isWindows ? 'Alt+f4' : 'CmdOrCtrl+Q',
          click() {
            app.quit();
          },
        },
        {
          label: 'Reload',
          accelerator: isWindows ? 'Ctrl+R' : 'CmdOrCtrl+R',
          click() {
            win.reload();
          },
        },
      ],
    },
    {
      label: 'Edit',
      submenu: [
        {
          label: 'Config Form Json',
          click() {
            openFormConfig();
          },
        },
        {
          label: 'Open developer tools',
          click() {
            win.openDevTools();
          },
        },
      ],
    },
    {
      label: 'Deploy',
      submenu: [
        {
          label: 'Diagnostic',
          accelerator: isWindows ? 'Alt+S' : 'CmdOrCtrl+S',
          click() {
            alert('TODO DIAGNOSTIC');
          },
        },
        {
          type: 'separator',
        },
        {
          label: 'Submit to review',
          click() {
            alert('SUBMIT TO REVIEW TODO');
          },
        },
      ],
    },
  ];

  const menu = Menu.buildFromTemplate(template);
  Menu.setApplicationMenu(menu);
}
