const fs = require('fs');
const { ipcRenderer } = require('electron');
const CssSelectorGenerator = require('css-selector-generator').CssSelectorGenerator;
const { ipcEvents } = require('../constants');
const { domReady, injectJquery, getPosition } = require('../helpers');
const ElementSelector = require('../elementSelector/main');

const elementSelector = new ElementSelector();

// Alias for AB Tasty tracking global function
window.ABTastyClickTracking = () => console.log('window.ABTastyClickTracking');
window.ABTastyEvent = () => console.log('window.ABTastyEvent');

// jQuery attempts to hook into things on the document object.
// So we need that the document is loaded before injecting jQuery.
// TODO: to remove when jQuery will no longer be needed for widgets.
domReady(() => injectJquery());

// SELECT ELEMENT
ipcRenderer.on(ipcEvents.selectElement, () => {
  elementSelector.append();
  document.body.addEventListener('mouseover', e => {
    elementSelector.hover(e);
  });
  document.body.addEventListener('click', function handleClickOnSelector(event) {
    event.preventDefault();
    document.body.removeEventListener('mouseover', () => elementSelector.hover());
    elementSelector.remove();
    const selectorGenerator = new CssSelectorGenerator();
    const selector = selectorGenerator.getSelector(event.target);
    document.body.removeEventListener('click', handleClickOnSelector);
    ipcRenderer.sendToHost(ipcEvents.selectElementComplete, selector);
  });
});
