/* global rootPath */
const path = require('path');
const fs = require('fs');
const { exec } = require('child_process');

const log = console.log;

function isValidWidget(widgetPath) {
  const pathExists = fs.existsSync(widgetPath) === true;
  const packageFilePath = path.join(widgetPath, 'package.json');
  const packageFileExists = fs.existsSync(packageFilePath) === true;

  if (!pathExists) {
    log('Invalid widget path.');
    return false;
  }
  if (packageFileExists) {
    const pkg = JSON.parse(fs.readFileSync(packageFilePath, 'utf8'));
    const mainFilePath = path.join(widgetPath, pkg.main);
    const formFilePath = path.join(widgetPath, pkg.form);
    const mainFileExists = fs.existsSync(mainFilePath) === true;
    const formFileExists = fs.existsSync(formFilePath) === true;
    if (!mainFileExists) {
      log(`Main file \`${pkg.main}\` not found.`);
      log(
        'Please make sure that you\'ve put a correct path in the main property in package.json (eg. "main": "dist/main.js").'
      );
      return false;
    }
    if (!formFileExists) {
      log(`Form file \`${pkg.form}\` not found.`);
      log(
        'Please make sure that you\'ve put a correct path in the form property in package.json (eg. "form": "dist/form.js").'
      );
      return false;
    }
  } else {
    log('package.json not found.');
    return false;
  }

  return true;
}

function launchElectron(widgetPath, url) {
  const fullWidgetPath = path.join(process.cwd(), widgetPath);
  const cmd = `"${require(`${rootPath}/node_modules/electron/index.js`)}" "${__dirname}/app/app.js" "${url}" ${fullWidgetPath}`;
  const electronProcess = exec(
    cmd,
    {
      shell: true,
    },
    (error, stdout, stderr) => {
      if (error) {
        console.error(`exec error: ${error}`);
        return;
      }
      log(`stdout: ${stdout}`);
      log(`stderr: ${stderr}`);
    }
  );

  electronProcess.stdout.on('data', data => {
    log(data.toString());
  });
  electronProcess.stdout.on('exit', exitCode => {
    log('exit', exitCode);
  });
}

function main(widgetPath = './', url = 'https://www.abtasty.com/') {
  if (isValidWidget(widgetPath)) {
    launchElectron(widgetPath, url);
  }
}

module.exports = main;
